// Reference Navigate.jQuery with $ and Navigate with $$
(function($, $$) {

function Widget(wid) {
  this.wid = wid;
  /**
   * Allow plugins to dynamically change the title of any widget.
   *
   * @param wid
   *   An integer containing the ID of the desired widget.
   * @param title
   *   A string containing the new title to be set.
   *
   * @return
   *   The previous widget's title, or FALSE if the widget does not exist.
   */
  this.changeTitle = function(title) {
    var widgetTitle = this.get().find('.navigate-widget-title');
    if (widgetTitle.length) {
      var previousTitle = $(widgetTitle).text();
      widgetTitle.animate({ opacity: 0 }, { queue: false, duration: 400, complete: function(){
        widgetTitle.text(title);
        widgetTitle.animate({ opacity: 1 }, { queue: false, duration: 400});
      }});
      if ($$.settings.customize) {
        $(widgetTitle).addClass('customize');
      }
      else {
        $(widgetTitle).removeClass('customize');
      }
      $$.execute({ event: 'changeTitle' });
      return previousTitle;
    }
    else {
      return false;
    }
  },
  this.customizeTitle = function(wid) {
    if (wid) {
      this.wid = wid;
    }
    var header = $(this.get()).find('.navigate-widget-header');
    var title = $(header).find('.navigate-widget-title');
    var oldTitle = title.text();
    var newTitle = '';
    $(title).hide();
    var input = $(header).find('.navigate-title-input');
    if (!input.length) {
      input = $('<input type="text"/>').addClass('navigate-title-input').val(oldTitle);
      $(header).prepend(input);
    }
    $(input).focus().select();
    $(input).blur(function(){
      $(title).show();
      $(this).remove();
    });
    $(input).bind('keyup', function(e) {
      // Enter or Esc pressed
      if (e.which == 13 || e.keyCode == 13 || e.which == 27 || e.keyCode == 27) {
        newTitle = $(this).val();
        if (newTitle == '') { newTitle = oldTitle; }
        if (oldTitle != newTitle) {
          // Save new title
          if (e.which == 13 || e.keyCode == 13) {
            $$.variableSet('title', newTitle, wid);
          }
          // Cancel
          else if (e.which == 27 || e.keyCode == 27) {
            newTitle = oldTitle;
          }
        }
        // Show Title
        $(title).text(newTitle).show();
        $(input).remove();
      }
    });
  }
  /**
   * Determine if a widget exists
   *
   * @param wid
   *   An integer containing the ID of the desired widget.
   *
   * @return
   *   TRUE if the widget exists or FALSE if the widget does not exist.
   */
  this.exists = function(wid) {
    if (wid) {
      this.wid = wid;
    }
    return $('#navigate-widget-' + this.wid).length ? true : false;
  },
  /**
  * Find a widget or set of widgets.
  *
  * @param {number|string|object|array} data
  *   Data is processed in the following order to determine the widget(s) that it should return:
  *     - {number}
  *       - {wid}:      A number containing the widget ID 
  *     - {string}
  *       - {wid}:      A string containing the number of a widget ID.
  *       - {plugin}:   A string containing the plugin name. This will return all widgets that were created by a certain plugin type.
  *     - {object}
  *       - {element}:  A DOM Element, Document or jQuery element. This will return the widget the element is a child of.
  *     - {array}
  *       - {number|string|object}:   An array of the previously accepted types. This is not recursive, do not nest arrays.
  * @return
  *   jQuery object of the widget(s) or FALSE if no critera match.
  *
  * Example usage:
  * @code
  *   // Returns the widget: 1.
  *   $$.getWidget(1);                              
  *   // Same as above, but with quotes. Either works.
  *   $$.getWidget('1');            
  *   // Returns the widgets: 1, 4, 5 and 6                
  *   $$.getWidget([ 1, 4, 5, 6 ]);     
  *   // Returns all widgets of the navigate_menu plugin type                 
  *   $$.getWidget('navigate_menu');
  *   // Returns the widget that this callback button is a child of.
  *   $$.getWidget($('.some-callback-button'));   
  *   // Returns the widgets: 3, all widgets that are navigate_search and the widget that the callback button is a child of.
  *   $$.getWidget([ 3, 'navigate_search', $('.yet-another-callback') ])
  * @endcode
  */
  this.get = function(data) {
    if (data == undefined) {
      data = this.wid; 
    }
    var widgets = [];
    if (!$$.isArray(data)) {
      data = [data];
    }
    for (i in data) {
      if ($$.isString(data[i]) || $$.isNumber(data[i])) {
        if (this.exists(data[i])) {
          $('#navigate-widget-'+data[i]).each(function(){
            widgets.push(this);
          });
        }
        else if ($$.pluginExists(data[i])) {
          $('navigate[plugin=' + data[i] + ']', '.navigate-widget').parents('.navigate-widget').each(function(){
            widgets.push(this);
          });
        }
        else {
          var element = $$.closest('.navigate-widget', data[i]);
          if (element) {
            element.each(function(){
              widgets.push(this);
            });
          }
        }
      }
      else if ($$.isObject(data[i])) {
        var element = $$.closest('.navigate-widget', data[i]);
        if (element) {
          element.each(function(){
            widgets.push(this);
          });
        }
      }
    }
    if (widgets.length) {
      return $($.unique(widgets));
    }
    else {
      return false;
    }
  }
  if (this.wid != undefined) {
    return this;
  }
  else {
    return false;
  }
}

$$.widget = function(wid) {
  var widget = Widget(wid);
  return widget;
}
$$.widget.prototype = new Widget();

})(Navigate.jQuery, Navigate);
